import { ChangeDetectorRef, Component, forwardRef, Input } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { keyBoardHelper } from '../utils/keyboard-utils';
function fillLeft(num) {
    return (num || 0) < 10 ? "0" + num : "" + num;
}
var ARROW_DOWN = 'ArrowDown';
var ARROW_UP = 'ArrowUp';
var supportKeyType = [ARROW_UP, ARROW_DOWN];
var TimePickerModel = (function () {
    function TimePickerModel(hour, minute, second) {
        this.hour = hour;
        this.minute = minute;
        this.second = second;
    }
    TimePickerModel.prototype.toString = function () {
        return fillLeft(this.hour) + ":" + fillLeft(this.minute) + ":" + fillLeft(this.second);
    };
    return TimePickerModel;
}());
export { TimePickerModel };
var BACKSPACE_KEY_CODE = 8;
var ENTER_KEY_CODE = 13;
var TAB_KEY_CODE = 9;
var SUPPORTED_KEY_CODE = [
    BACKSPACE_KEY_CODE,
    ENTER_KEY_CODE,
    TAB_KEY_CODE
];
export var TIME_UNIT = {
    HOUR: 'hour',
    MINUTE: 'minute',
    SECOND: 'second',
};
export var MAX_TIME_RANGE = {
    hour: 23,
    minute: 59,
    second: 59,
};
var TimePickerComponent = (function () {
    function TimePickerComponent(changeDetectorRef) {
        this.changeDetectorRef = changeDetectorRef;
        this.showSeconds = true;
        this.minTime = new TimePickerModel(0, 0, 0);
        this.maxTime = new TimePickerModel(23, 59, 59);
        this.hour = '00';
        this.minute = '00';
        this.second = '00';
        this.timeUnit = TIME_UNIT;
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
    }
    TimePickerComponent.prototype.ngOnInit = function () {
        this.initConditionDate();
    };
    TimePickerComponent.prototype.initConditionDate = function () {
        this.minDate = this.getDateByTime(this.minTime);
        this.maxDate = this.getDateByTime(this.maxTime);
    };
    TimePickerComponent.prototype.getDateByTime = function (time) {
        return new Date((new Date()).setHours(time.hour || 0, time.minute || 0, time.second || 0));
    };
    TimePickerComponent.prototype.getValue = function (value, defaultValue) {
        if (value || value === 0) {
            return value;
        }
        return defaultValue;
    };
    TimePickerComponent.prototype.getCurrentTimestamp = function (time) {
        var hour = this.getValue(time.hour, parseInt(this.hour, 10));
        var minute = this.getValue(time.minute, parseInt(this.minute, 10));
        var second = this.getValue(time.second, parseInt(this.second, 10));
        return (new Date()).setHours(hour, minute, second);
    };
    TimePickerComponent.prototype.writeValue = function (value) {
        if (value) {
            this.hour = fillLeft(value.hour || 0);
            this.minute = fillLeft(value.minute || 0);
            this.second = fillLeft(value.second || 0);
            this.changeDetectorRef.markForCheck();
        }
    };
    TimePickerComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    TimePickerComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    TimePickerComponent.prototype.setDisabledState = function (disabled) {
        this.disabled = disabled;
        this.changeDetectorRef.markForCheck();
    };
    TimePickerComponent.prototype.updateTimeByMargin = function (key, marginDate) {
        switch (key) {
            case TIME_UNIT.HOUR:
                this.hour = fillLeft(marginDate[TIME_UNIT.HOUR]);
                this.minute = fillLeft(marginDate[TIME_UNIT.MINUTE]);
                if (marginDate[TIME_UNIT.SECOND]) {
                    this.second = fillLeft(marginDate[TIME_UNIT.SECOND]);
                }
                break;
            case TIME_UNIT.MINUTE:
                this.minute = fillLeft(marginDate[TIME_UNIT.MINUTE]);
                if (marginDate[TIME_UNIT.SECOND]) {
                    this.second = fillLeft(marginDate[TIME_UNIT.SECOND]);
                }
                break;
            case TIME_UNIT.SECOND:
                this.second = fillLeft(marginDate[TIME_UNIT.SECOND]);
                break;
        }
    };
    TimePickerComponent.prototype.modifyTimeByKey = function (value, key) {
        if (isNaN(value)) {
            if (this.getCurrentTimestamp((_a = {}, _a[key] = 0, _a)) < this.minDate.getTime()) {
                this.updateTimeByMargin(key, this.minTime);
            }
            else if (this.getCurrentTimestamp((_b = {}, _b[key] = 0, _b)) > this.maxDate.getTime()) {
                this.updateTimeByMargin(key, this.maxTime);
            }
            else {
                this.updateTimeByMargin(key, this.minTime);
            }
        }
        else if (value < 0) {
            this[key] = fillLeft(0) || fillLeft(this.minTime[key]);
        }
        else if (value > MAX_TIME_RANGE[key] || this.getCurrentTimestamp((_c = {}, _c[key] = value, _c)) > this.maxDate.getTime()) {
            this.updateTimeByMargin(key, this.maxTime);
        }
        else if (this.getCurrentTimestamp((_d = {}, _d[key] = value, _d)) < this.minDate.getTime()) {
            this.updateTimeByMargin(key, this.minTime);
        }
        else {
            this[key] = fillLeft(value);
        }
        var _a, _b, _c, _d;
    };
    TimePickerComponent.prototype.onTimeChange = function (key) {
        this.onTouched();
        this.modifyTimeByKey(parseInt(this[key], 10), key);
        this.onModelChange();
    };
    TimePickerComponent.prototype.onModelChange = function () {
        this.onChange(this.getTimePickerModel());
    };
    TimePickerComponent.prototype.getTimePickerModel = function () {
        var model = new TimePickerModel(parseInt(this.hour, 10), parseInt(this.minute, 10));
        if (this.showSeconds) {
            model.second = parseInt(this.second, 10);
        }
        return model;
    };
    TimePickerComponent.prototype.isSafetyKeyPress = function (keyType, target, key) {
        var parsedTarget = parseInt(target, 10);
        return !(keyType === ARROW_DOWN && parsedTarget < 1) &&
            ((keyType === ARROW_DOWN && key === TIME_UNIT.HOUR && parsedTarget <= MAX_TIME_RANGE.hour) ||
                (keyType === ARROW_DOWN && (key === TIME_UNIT.MINUTE ||
                    key === TIME_UNIT.SECOND) && parsedTarget <= MAX_TIME_RANGE.minute) ||
                (keyType === ARROW_UP && key === TIME_UNIT.HOUR && parsedTarget < MAX_TIME_RANGE.hour) ||
                (keyType === ARROW_UP && (key === TIME_UNIT.MINUTE ||
                    key === TIME_UNIT.SECOND) && parsedTarget < MAX_TIME_RANGE.minute));
    };
    TimePickerComponent.prototype.isSupportedKeyPress = function (event) {
        return SUPPORTED_KEY_CODE.indexOf(event.keyCode) > -1 ||
            keyBoardHelper.numberKeyBoard(event) ||
            keyBoardHelper.smallKeyBoard(event) ||
            keyBoardHelper.arrowKeyBoard(event);
    };
    TimePickerComponent.prototype.handleKeyEvent = function (event, type) {
        if (!this.isSupportedKeyPress(event)) {
            event.preventDefault();
            return;
        }
        var keyEventType = event.code;
        if (supportKeyType.indexOf(keyEventType) === -1) {
            return;
        }
        this.modifyTimeByKeyPressEvent(type, keyEventType);
    };
    TimePickerComponent.prototype.modifyTimeByKeyPress = function (key, type, step) {
        if (this.isSafetyKeyPress(type, this[key], key)) {
            this[key] = fillLeft(parseInt(this[key], 10) + step);
        }
    };
    TimePickerComponent.prototype.modifyTimeByKeyPressEvent = function (type, keyEventType) {
        var step = 1;
        if (keyEventType === ARROW_DOWN) {
            step = -1;
        }
        switch (type) {
            case this.timeUnit.HOUR:
                this.modifyTimeByKeyPress(TIME_UNIT.HOUR, keyEventType, step);
                break;
            case this.timeUnit.MINUTE:
                this.modifyTimeByKeyPress(TIME_UNIT.MINUTE, keyEventType, step);
                break;
            case this.timeUnit.SECOND:
                this.modifyTimeByKeyPress(TIME_UNIT.SECOND, keyEventType, step);
                break;
        }
    };
    TimePickerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-time-picker',
                    template: "\n    <form class=\"form-inline time-picker\"><div class=\"form-group\"><input #hours [disabled]=\"disabled\" name=\"hours\" class=\"form-control\" maxlength=\"2\" max=\"23\" min=\"0\" autocomplete=\"off\" [(ngModel)]=\"hour\" (keydown)=\"handleKeyEvent($event, timeUnit.HOUR)\" (blur)=\"onTimeChange(timeUnit.HOUR)\"></div><span>:</span><div class=\"form-group\"><input #minutes [disabled]=\"disabled\" name=\"minutes\" class=\"form-control\" maxlength=\"2\" max=\"59\" min=\"0\" autocomplete=\"off\" [(ngModel)]=\"minute\" (keydown)=\"handleKeyEvent($event, timeUnit.MINUTE)\" (blur)=\"onTimeChange(timeUnit.MINUTE)\"></div><ng-container *ngIf=\"showSeconds\"><span>:</span><div class=\"form-group\"><input #seconds [disabled]=\"disabled\" name=\"seconds\" class=\"form-control\" max=\"59\" min=\"0\" maxlength=\"2\" autocomplete=\"off\" [(ngModel)]=\"second\" (keydown)=\"handleKeyEvent($event, timeUnit.SECOND)\" (blur)=\"onTimeChange(timeUnit.SECOND)\"></div></ng-container></form>\n  ",
                    exportAs: 'timePicker',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return TimePickerComponent; }),
                            multi: true
                        }
                    ]
                },] },
    ];
    TimePickerComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
    ]; };
    TimePickerComponent.propDecorators = {
        "showSeconds": [{ type: Input },],
        "minTime": [{ type: Input },],
        "maxTime": [{ type: Input },],
    };
    return TimePickerComponent;
}());
export { TimePickerComponent };
